<?php
/**
 * Gravatar 头像缓存插件SAE专版(基于GravatarCache修改开发)
 * 
 * @package GravatarCache 
 * @author YoviSun
 * @version 3.0.0
 * 
 * 历史版本
 *
 * version 3.0.0 at 2015-01-17
 * 1）由于gravatar的链接地址命名规则有了更新，致使该插件无法使用，本次已经修复
 * 2）修复了禁用插件时删除Storage出错的bug
 *
 * version 2.01 原版
 * @link http://www.yovisun.com
 *
 */

class GravatarCache_Plugin implements Typecho_Plugin_Interface
{
    /**
     * 激活插件方法,如果激活失败,直接抛出异常
     * 
     * @access public
     * @return void
     * @throws Typecho_Plugin_Exception
     */
    public static function activate()
    {
        Typecho_Plugin::factory('Widget_Abstract_Comments')->gravatar = array('GravatarCache_Plugin', 'getGravatar');
    }
    
    /**
     * 禁用插件方法,如果禁用失败,直接抛出异常
     * 
     * @static
     * @access public
     * @return void
     * @throws Typecho_Plugin_Exception
     */
    public static function deactivate()
	{
		self::deleteFile();
	}
    
    /**
     * 获取插件配置面板
     * 
     * @access public
     * @param Typecho_Widget_Helper_Form $form 配置面板
     * @return void
     */
    public static function config(Typecho_Widget_Helper_Form $form)
    {
		$timeCache = new Typecho_Widget_Helper_Form_Element_Text('timeCache', NULL, '14', _t('缓存时间'),_t('缓存时间，默认 14天'));
		$timeCache->input->setAttribute('class', 'mini');
		$form->addInput($timeCache->addRule('required', _t('必须填写缓存时间'))->addRule('isInteger', _t('缓存时间必须是整数')));
		
		$avatarSize = new Typecho_Widget_Helper_Form_Element_Text('avatarSize', NULL, '40', _t('图像大小'),_t('前台显示的图片的尺寸大小，正方形'));
        $avatarSize->input->setAttribute('class', 'mini');
        $form->addInput($avatarSize->addRule('required', _t('必须填写图像大小'))
        ->addRule('isInteger', _t('图像大小必须是纯数字')));
		
		$domain = new Typecho_Widget_Helper_Form_Element_Text('domain', null, 'typecho', _t('存放Storage名称'), _t('存放Storage名称！'));
		$form->addInput($domain->addRule('required', _t('必须填写存储Storage名称')));
		
		$dir = new Typecho_Widget_Helper_Form_Element_Text('dir', null, 'avatar', _t('存放路径'), _t('缓存头像存放的路径！如avatar'));
		$form->addInput($dir->addRule('required', _t('必须填写缓存目录')));
		
		$delCache= new Typecho_Widget_Helper_Form_Element_Radio( 'delCache', array( 'delY' => '是', 'delN' => '否' ), 'delY', '删除缓存图像',_t('禁用插件时是否删除缓存头像') );
        $form->addInput($delCache);
		
		return _t('请到插件配置里设置相应选项');
	}
    
    /**
     * 个人用户的配置面板
     * 
     * @access public
     * @param Typecho_Widget_Helper_Form $form
     * @return void
     */
    public static function personalConfig(Typecho_Widget_Helper_Form $form){}
    
    /**
     * 插件实现方法
     * 
     * @access public
     * @return void
     */
    public static function getGravatar($size, $rating, $default, $comments)
    {
	$option = Typecho_Widget::widget('Widget_Options')->plugin('GravatarCache');
	$s = new SaeStorage();
	$avatar_id = empty($comments->mail) ? '00000000000000000000000000000000' : md5( strtolower( $comments->mail ) );
	
		if($comments->request->isSecure()){
				$host = 'https://secure.gravatar.com';
			}else{
				 $host = 'http://gravatar.com';

			}
	$avatar_url = $host.'/avatar/'.$avatar_id.'?s='.$options->avatarSize .'&r=G&d=';
	
			
	
	$avatar_file = $option->dir .'/' . $avatar_id . '.png';
	$avatar_cache = "'now plus ".$option->$timeCache ." day'";
	if(!$s->fileExists($option->domain, $avatar_file))
	  {
		  $content = file_get_contents($avatar_url);
		  $attr = array('expires' => 'now plus 14 day');
		  //$attr = array('expires' => $avatar_cache );
		  $result = $s->write($option->domain, $avatar_file, $content, -1, $attr);
		  if ($result != true)
		  var_dump($s->errno(), $s->errmsg());
	  }
	$avatar = $s->getUrl($option->domain , $avatar_file);
	echo '<img class="avatar" src="'.$avatar.'" alt="'.$comments->author.'" width="'.$option->avatarSize.'" height="'.$option->avatarSize.'" />';
    }
	/**
     * 禁用插件时同时删除缓存头像
     * 
     * @access public
     * @return void
     */
	public static function deleteFile()
	{
		
		$option = Typecho_Widget::widget('Widget_Options')->plugin('GravatarCache');
		$s = new SaeStorage();
		if( $option->delCache == 'delY' ){          
            while ( $ret = $s->getList($option->domain , $option->dir, 100, 0 ) ){
                foreach($ret as $file)
                    $s->delete($option->domain,$file);
            }
        }
	}
    
}